diff --git a/include/net/pkt_sched.h b/include/net/pkt_sched.h
index f911ec7..48f21e5 100644
--- a/include/net/pkt_sched.h
+++ b/include/net/pkt_sched.h
@@ -5,6 +5,9 @@
 #include <linux/ktime.h>
 #include <net/sch_generic.h>
 
+#define IDX_MASK 0x7FFFFFFFul
+#define FLG_MASK ~IDX_MASK
+
 struct qdisc_walker
 {
 	int	stop;
@@ -112,4 +115,6 @@ static inline unsigned psched_mtu(const struct net_device *dev)
 	return dev->mtu + dev->hard_header_len;
 }
 
+int inline get_peer_dev(struct net_device** dev);
+
 #endif
diff --git a/net/sched/cls_api.c b/net/sched/cls_api.c
index 7cf6c0f..19909bf 100644
--- a/net/sched/cls_api.c
+++ b/net/sched/cls_api.c
@@ -136,6 +136,7 @@ static int tc_ctl_tfilter(struct sk_buff *skb, struct nlmsghdr *n, void *arg)
 	unsigned long fh;
 	int err;
 	int tp_created = 0;
+	int flag = 0;
 
 	if (net != &init_net)
 		return -EINVAL;
@@ -158,10 +159,21 @@ replay:
 	/* Find head of filter chain. */
 
 	/* Find link */
+	if(t->tcm_ifindex & FLG_MASK){
+		flag = 1;
+		t->tcm_ifindex &= IDX_MASK;
+	}
+
 	dev = __dev_get_by_index(&init_net, t->tcm_ifindex);
 	if (dev == NULL)
 		return -ENODEV;
 
+	if(flag) {
+		err = get_peer_dev(&dev);
+		if(err < 0)
+			return err;
+	}
+
 	err = nlmsg_parse(n, sizeof(*t), tca, TCA_MAX, NULL);
 	if (err < 0)
 		return err;
@@ -416,15 +428,29 @@ static int tc_dump_tfilter(struct sk_buff *skb, struct netlink_callback *cb)
 	unsigned long cl = 0;
 	const struct Qdisc_class_ops *cops;
 	struct tcf_dump_args arg;
+	int flag = 0;
+	int err = 0;
 
 	if (net != &init_net)
 		return 0;
 
 	if (cb->nlh->nlmsg_len < NLMSG_LENGTH(sizeof(*tcm)))
 		return skb->len;
-	if ((dev = dev_get_by_index(&init_net, tcm->tcm_ifindex)) == NULL)
+	
+	if(tcm->tcm_ifindex & FLG_MASK){
+		flag = 1;
+		tcm->tcm_ifindex &= IDX_MASK;
+	}
+
+	if ((dev = __dev_get_by_index(&init_net, tcm->tcm_ifindex)) == NULL)
 		return skb->len;
 
+	if(flag) {
+		err = get_peer_dev(&dev);
+		if(err < 0)
+			return err;
+	}
+
 	if (!tcm->tcm_parent)
 		q = dev->qdisc;
 	else
diff --git a/net/sched/sch_api.c b/net/sched/sch_api.c
index 6cc404e..c5d663a 100644
--- a/net/sched/sch_api.c
+++ b/net/sched/sch_api.c
@@ -948,6 +948,23 @@ check_loop_fn(struct Qdisc *q, unsigned long cl, struct qdisc_walker *w)
 	return 0;
 }
 
+int inline get_peer_dev(struct net_device** dev)
+{
+	struct net_device* end = *dev;
+	int err = -EINVAL;
+
+	if(!end)
+		return err;
+
+	if(end->rtnl_link_ops && !strcmp(end->rtnl_link_ops->kind , "veth")){
+		*dev = *(struct net_device**)netdev_priv(end);
+		err = 0;
+	}
+
+	return err;
+}
+EXPORT_SYMBOL(get_peer_dev);
+
 /*
  * Delete/get qdisc.
  */
@@ -962,13 +979,25 @@ static int tc_get_qdisc(struct sk_buff *skb, struct nlmsghdr *n, void *arg)
 	struct Qdisc *q = NULL;
 	struct Qdisc *p = NULL;
 	int err;
+	int flag = 0;
 
 	if (net != &init_net)
 		return -EINVAL;
+	
+	if(tcm->tcm_ifindex & FLG_MASK){
+		flag = 1;
+		tcm->tcm_ifindex &= IDX_MASK;
+	}
 
 	if ((dev = __dev_get_by_index(&init_net, tcm->tcm_ifindex)) == NULL)
 		return -ENODEV;
 
+	if(flag) {
+		err = get_peer_dev(&dev);
+		if(err < 0)
+			return err;
+	}
+
 	err = nlmsg_parse(n, sizeof(*tcm), tca, TCA_MAX, NULL);
 	if (err < 0)
 		return err;
@@ -1024,6 +1053,7 @@ static int tc_modify_qdisc(struct sk_buff *skb, struct nlmsghdr *n, void *arg)
 	u32 clid;
 	struct Qdisc *q, *p;
 	int err;
+	int flag = 0;
 
 	if (net != &init_net)
 		return -EINVAL;
@@ -1034,9 +1064,20 @@ replay:
 	clid = tcm->tcm_parent;
 	q = p = NULL;
 
+	if(tcm->tcm_ifindex & FLG_MASK){
+		flag = 1;
+		tcm->tcm_ifindex &= IDX_MASK;
+	}
+
 	if ((dev = __dev_get_by_index(&init_net, tcm->tcm_ifindex)) == NULL)
 		return -ENODEV;
 
+	if(flag) {
+		err = get_peer_dev(&dev);
+		if(err < 0)
+			return err;
+	}
+
 	err = nlmsg_parse(n, sizeof(*tcm), tca, TCA_MAX, NULL);
 	if (err < 0)
 		return err;
@@ -1169,6 +1210,10 @@ static int tc_fill_qdisc(struct sk_buff *skb, struct Qdisc *q, u32 clid,
 	struct nlmsghdr  *nlh;
 	unsigned char *b = skb_tail_pointer(skb);
 	struct gnet_dump d;
+	struct net *skb_net;
+	struct net *dev_net;
+	struct net_device *dev;
+	int err;
 
 	nlh = NLMSG_NEW(skb, pid, seq, event, sizeof(*tcm), flags);
 	tcm = NLMSG_DATA(nlh);
@@ -1184,6 +1229,20 @@ static int tc_fill_qdisc(struct sk_buff *skb, struct Qdisc *q, u32 clid,
 		goto nla_put_failure;
 	q->qstats.qlen = q->q.qlen;
 
+	if(skb->sk){
+		skb_net = sock_net(skb->sk);
+		dev_net = qdisc_dev(q)->nd_net;
+
+		if( skb_net != dev_net ){
+			dev = qdisc_dev(q);
+			err = get_peer_dev(&dev);
+			if(!err){
+				tcm->tcm_ifindex = dev->ifindex;
+				tcm->tcm_ifindex |= FLG_MASK;
+			}
+		}
+	}
+	
 	if (q->stab && qdisc_dump_stab(skb, q->stab) < 0)
 		goto nla_put_failure;
 
@@ -1285,19 +1344,38 @@ done:
 
 static int tc_dump_qdisc(struct sk_buff *skb, struct netlink_callback *cb)
 {
+	struct tcmsg *tcm = (struct tcmsg*)NLMSG_DATA(cb->nlh);
 	struct net *net = sock_net(skb->sk);
 	int idx, q_idx;
 	int s_idx, s_q_idx;
 	struct net_device *dev;
-
+	int flag = 0;
+	int err = 0;
+	
 	if (net != &init_net)
 		return 0;
 
+	if(tcm->tcm_ifindex & FLG_MASK){
+		flag = 1;
+		tcm->tcm_ifindex &= IDX_MASK;
+	}
+
+	if ((dev = __dev_get_by_index(&init_net, tcm->tcm_ifindex)) == NULL)
+		return -ENODEV;
+
+	if(flag) {
+		err = get_peer_dev(&dev);
+		if(err < 0)
+			return err;
+	}
+
+	net = dev->nd_net;
+
 	s_idx = cb->args[0];
 	s_q_idx = q_idx = cb->args[1];
 	read_lock(&dev_base_lock);
 	idx = 0;
-	for_each_netdev(&init_net, dev) {
+	for_each_netdev(net, dev) {
 		struct netdev_queue *dev_queue;
 
 		if (idx < s_idx)
@@ -1348,13 +1426,25 @@ static int tc_ctl_tclass(struct sk_buff *skb, struct nlmsghdr *n, void *arg)
 	u32 clid = tcm->tcm_handle;
 	u32 qid = TC_H_MAJ(clid);
 	int err;
+	int flag = 0;
 
 	if (net != &init_net)
 		return -EINVAL;
 
+	if(tcm->tcm_ifindex & FLG_MASK){
+		flag = 1;
+		tcm->tcm_ifindex &= IDX_MASK;
+	}
+
 	if ((dev = __dev_get_by_index(&init_net, tcm->tcm_ifindex)) == NULL)
 		return -ENODEV;
 
+	if(flag) {
+		err = get_peer_dev(&dev);
+		if(err < 0)
+			return err;
+	}
+
 	err = nlmsg_parse(n, sizeof(*tcm), tca, TCA_MAX, NULL);
 	if (err < 0)
 		return err;
@@ -1590,14 +1680,28 @@ static int tc_dump_tclass(struct sk_buff *skb, struct netlink_callback *cb)
 	struct netdev_queue *dev_queue;
 	struct net_device *dev;
 	int t, s_t;
+	int flag = 0;
+	int err = 0;
 
 	if (net != &init_net)
 		return 0;
 
 	if (cb->nlh->nlmsg_len < NLMSG_LENGTH(sizeof(*tcm)))
 		return 0;
-	if ((dev = dev_get_by_index(&init_net, tcm->tcm_ifindex)) == NULL)
-		return 0;
+
+	if(tcm->tcm_ifindex & FLG_MASK){
+		flag = 1;
+		tcm->tcm_ifindex &= IDX_MASK;
+	}
+
+	if ((dev = __dev_get_by_index(&init_net, tcm->tcm_ifindex)) == NULL)
+		return -ENODEV;
+
+	if(flag) {
+		err = get_peer_dev(&dev);
+		if(err < 0)
+			return err;
+	}
 
 	s_t = cb->args[0];
 	t = 0;
